==> enterprisemodules-ora_config-3.9.6.tar.gz: enterprisemodules-ora_config-3.9.6/lib/puppet/type/ora_service/preferred_instances.rb <==
#
# See the file "LICENSE" for the full license governing this code.
#
newproperty(:preferred_instances, :array_matching => :all) do
  include Puppet_X::EnterpriseModules::Oracle::ServiceProperty
  include EasyType::ArrayProperty
  include Puppet_X::EnterpriseModules::Oracle::Commands

  desc <<-EOD
    A list of preferred instances on which the service runs when the database is administrator managed.

    Here is an example on how to use this:

        ora_service{'my_service':
          ...
          prefered_instances => ['SID1', 'SID2'],
          ...
        }

    When an instance is specified that does not or not yes exist in the cluster, Puppet will
    ignore it.

    This is a cluster only property. On single node database this property will be ignored. If you use it,
    Puppet will issue a warning.

  EOD

  column_name 'Preferred instances'
  data_type("Optional[Variant[String[1],Array]]")

  defaultto do
    begin
      cluster_instances
    rescue RuntimeError
      # When the database doesn't exist yet, use a nil. Later we will fix the
      # default value back again.
      []
    end
  end

  def insync?(is)
    is = [] if is.to_s == 'absent'
    if cluster?
      is.sort == (should - unknown_instances).sort
    else
      Puppet.info "#{path}: #{name} has no effect on non clustered service." unless should == [resource.sid]
      true
    end
  end

  to_translate_to_resource do |raw_resource|
    value = super(raw_resource)
    value.nil? ? nil : value.to_s.split(',').map(&:strip)
  end

  on_create do
    apply('create')
  end

  on_modify do
    # modify is handled in after_modify
    nil
  end

  after_modify do
    if cluster?
      #
      # If it contains unknown instances at this point in time, filter them out
      #
      if resource[:available_instances].any?
        if unknown_instances?(resource[:available_instances])
          sane_avail_value = resource[:available_instances] - unknown_instances(resource[:available_instances])
          avail = provider.send('update_available_instances', sane_avail_value)
        else
          avail = provider.send('update_available_instances', resource[:available_instances])
        end
      end
      if unknown_instances?
        sane_value = value - unknown_instances
        if sane_value.any?
          Puppet.info "#{resource}: Skipping unknown instances #{unknown_instances.join(',')} continue with #{sane_value.join(',')}" unless @warned
          @warned = true
          pref = provider.send("update_#{self.class.name}", sane_value)
          srvctl("modify service -d #{sid.chop} -service #{resource[:service_name]} #{pref} #{avail}", :sid => sid)
        else
          fail "#{resource}: No known instances specified for property preferred_instances"
        end
      else
        pref = provider.send("update_#{self.class.name}", value)
        srvctl("modify service -d #{sid.chop} -service #{resource[:service_name]} #{pref} #{avail}", :sid => sid)
      end
    else
      Puppet.warning "#{resource}: preferred_instances has no effect on non clustered service." unless resource.preferred_instances == [resource.sid]
    end
  end

  private

  def apply(type)
    if cluster?
      #
      # If it contains unknown instances at this point in time, filter them out
      #
      if unknown_instances?
        sane_value = value - unknown_instances
        if sane_value.any?
          Puppet.info "#{resource}: Skipping unknown instances #{unknown_instances.join(',')} continue with #{sane_value.join(',')}" unless @warned
          @warned = true
          provider.send("#{type}_#{self.class.name}", sane_value)
        else
          fail "#{resource}: No known instances specified for property preferred_instances"
        end
      else
        provider.send("#{type}_#{self.class.name}", value) if value.any?
      end
    else
      Puppet.warning "#{resource}: preferred_instances has no effect on non clustered service." unless resource.preferred_instances == [resource.sid]
    end
  end

  #
  # If the value is empty, use all current instances as default
  #
  def requested_value
    value.empty? ? cluster_instances : value
  end

  def unknown_instances?(val = value)
    unknown_instances(val).any?
  end

  def unknown_instances(val = value)
    if val == value
      requested_value - cluster_instances
    else
      val - cluster_instances
    end
  end

  def known_instances
    requested_value & cluster_instances
  end
end

==> enterprisemodules-ora_config-3.9.7.tar.gz: enterprisemodules-ora_config-3.9.7/lib/puppet/type/ora_service/preferred_instances.rb <==
#
# See the file "LICENSE" for the full license governing this code.
#
newproperty(:preferred_instances, :array_matching => :all) do
  include Puppet_X::EnterpriseModules::Oracle::ServiceProperty
  include EasyType::ArrayProperty
  include Puppet_X::EnterpriseModules::Oracle::Commands

  desc <<-EOD
    A list of preferred instances on which the service runs when the database is administrator managed.

    Here is an example on how to use this:

        ora_service{'my_service':
          ...
          prefered_instances => ['SID1', 'SID2'],
          ...
        }

    When an instance is specified that does not or not yes exist in the cluster, Puppet will
    ignore it.

    This is a cluster only property. On single node database this property will be ignored. If you use it,
    Puppet will issue a warning.

  EOD

  column_name 'Preferred instances'
  data_type("Optional[Variant[String[1],Array]]")

  defaultto do
    begin
      cluster_instances
    rescue RuntimeError
      # When the database doesn't exist yet, use a nil. Later we will fix the
      # default value back again.
      []
    end
  end

  def insync?(is)
    is = [] if is.to_s == 'absent'
    if cluster?
      is.sort == (should - unknown_instances).sort
    else
      Puppet.info "#{path}: #{name} has no effect on non clustered service." unless should == [resource.sid]
      true
    end
  end

  to_translate_to_resource do |raw_resource|
    value = super(raw_resource)
    value.nil? ? nil : value.to_s.split(',').map(&:strip)
  end

  on_create do
    apply('create')
  end

  on_modify do
    # modify is handled in after_modify
    nil
  end

  after_modify do
    if cluster?
      #
      # If it contains unknown instances at this point in time, filter them out
      #
      if resource[:available_instances]
        if unknown_instances?(resource[:available_instances])
          sane_avail_value = resource[:available_instances] - unknown_instances(resource[:available_instances])
          avail = provider.send('update_available_instances', sane_avail_value)
        else
          avail = provider.send('update_available_instances', resource[:available_instances])
        end
      end
      if pdb?(sid)
        container = configuration_value_for(sid,'contained_by')
      else
        container = sid
      end
      if unknown_instances?
        sane_value = value - unknown_instances
        if sane_value.any?
          Puppet.info "#{resource}: Skipping unknown instances #{unknown_instances.join(',')} continue with #{sane_value.join(',')}" unless @warned
          @warned = true
          pref = provider.send("update_#{self.class.name}", sane_value)
          srvctl("modify service -d #{container.chop} -service #{resource[:service_name]} #{pref} #{avail}", :sid => container)
        else
          fail "#{resource}: No known instances specified for property preferred_instances"
        end
      else
        pref = provider.send("update_#{self.class.name}", value)
        srvctl("modify service -d #{container.chop} -service #{resource[:service_name]} #{pref} #{avail}", :sid => container)
      end
    else
      Puppet.warning "#{resource}: preferred_instances has no effect on non clustered service." unless resource.preferred_instances == [resource.sid]
    end
  end

  private

  def apply(type)
    if cluster?
      #
      # If it contains unknown instances at this point in time, filter them out
      #
      if unknown_instances?
        sane_value = value - unknown_instances
        if sane_value.any?
          Puppet.info "#{resource}: Skipping unknown instances #{unknown_instances.join(',')} continue with #{sane_value.join(',')}" unless @warned
          @warned = true
          provider.send("#{type}_#{self.class.name}", sane_value)
        else
          fail "#{resource}: No known instances specified for property preferred_instances"
        end
      else
        provider.send("#{type}_#{self.class.name}", value) if value.any?
      end
    else
      Puppet.warning "#{resource}: preferred_instances has no effect on non clustered service." unless resource.preferred_instances == [resource.sid]
    end
  end

  #
  # If the value is empty, use all current instances as default
  #
  def requested_value
    value.empty? ? cluster_instances : value
  end

  def unknown_instances?(val = value)
    unknown_instances(val).any?
  end

  def unknown_instances(val = value)
    if val == value
      requested_value - cluster_instances
    else
      val - cluster_instances
    end
  end

  def known_instances
    requested_value & cluster_instances
  end
end
